@@ -214,7 +214,7 @@ server.bufsize = <integer>")
     ;;       SO_REUSEPORT (since 3.9)
     ;;       Allows mutiple servers to listen to the same socket port, say 8080.
     ((server multi)
-     #t
+     #f
      "This is the most significant feature of Ragnarok server core.
 Please remember that there're no threads in GNU Artanis.
 All the tasks are based on delimited continuations, this kind of design is the
@@ -241,6 +241,16 @@ server.pub = <string>")
      "Whether to use Linux specified sendfile interface.
 server.sendfile = <boolean>")
 
+    ((server mmapped)
+     #f
+     "Whether to use POSIX specific mmap for file I/O.
+server.mmapped = <boolean>")
+
+    ((server allowedmethods)
+     (HEAD GET POST PUT)
+     "The allowed HTTP methods.
+server.allowedmethods = <methods-list>")
+
     ((websocket maxpayload)
      ,(1- (ash 1 64))
      "The maximum payload size of WebSocket request in bytes. If it exceeds, then it
@@ -361,6 +371,11 @@ debug.monitor = <PATHs>")))
          (string->symbol (string-trim-both e)))
        (filter (lambda (s) (not (string-null? s))) (string-split x #\,))))
 
+(define-syntax-rule (->string-list x)
+  (map (lambda (e)
+         (string-trim-both e))
+       (filter (lambda (s) (not (string-null? s))) (string-split x #\,))))
+
 (define-syntax-rule (->none/str x)
   (case (string->symbol (string-downcase x))
     ((none null #{}#) "")
@@ -400,6 +415,11 @@ debug.monitor = <PATHs>")))
         'mysql
         d)))
 
+(define-syntax-rule (->methods ml)
+  (map (lambda (m)
+         (string->symbol (string-upcase m)))
+       (->string-list ml)))
+
 (define *pool-modes* '(increase fixed))
 (define-syntax-rule (->pool-mode x)
   (let ((p (string->symbol x)))
@@ -444,6 +464,8 @@ debug.monitor = <PATHs>")))
     (('websocket websocket) (conf-set! '(server websocket) (->bool websocket)))
     (('pub pub) (conf-set! '(server pub) (basename pub)))
     (('sendfile v) (conf-set! '(server sendfile) (->bool v)))
+    (('mmapped v) (conf-set! '(server mmapped) (->bool v)))
+    (('allowedmethods ml) (conf-set! '(server allowedmethods) (->methods ml)))
     (else (error parse-namespace-server "Config: Invalid item" item))))
 
 (define (parse-namespace-websocket item)
@@ -457,7 +479,7 @@ debug.monitor = <PATHs>")))
 
 (define (parse-namespace-host item)
   (match item
-    (('name name) (conf-set! '(host name) name))
+    (('name name) (conf-set! '(host name) (->none/boolean name)))
     (('family family) (conf-set! '(host family) (->symbol family)))
     (('addr addr) (conf-set! '(host addr) addr))
     (('port port) (conf-set! '(host port) (->integer port)))
@@ -596,8 +618,13 @@ debug.monitor = <PATHs>")))
 ;;       art work -c ./my.conf
 ;; And init-server should be called automatically.
 (define current-conf-file (make-parameter #f))
-(define (current-myhost)
-  (format #f "http://~a:~a" (get-conf '(host addr)) (get-conf '(host port))))
+(define* (current-myhost #:key (for-header? #f))
+  (let* ((host (get-conf '(host name)))
+         (real-host (if host host (get-conf '(host addr))))
+         (port (get-conf '(host port))))
+    (if for-header?
+        (cons real-host #f)
+        (format #f "http://~a:~a" (get-conf '(host addr)) port))))
 
 (define (init-config)
   (define conf-file (current-conf-file)) ; user specified config
