@@ -33,12 +33,13 @@
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module (ice-9 ftw)
-  #:use-module (srfi srfi-1)
-  #:use-module (srfi srfi-19)
+  #:use-module (ice-9 iconv)
   #:use-module (ice-9 local-eval)
   #:use-module (ice-9 receive)
   #:use-module (ice-9 q)
   #:use-module (ice-9 control)
+  #:use-module (srfi srfi-1)
+  #:use-module (srfi srfi-19)
   #:use-module (web http)
   #:use-module (web request)
   #:use-module (web response)
@@ -49,7 +50,7 @@
                           bytevector-u8-ref string->utf8 bytevector-length
                           make-bytevector bytevector-s32-native-ref bytevector?
                           define-record-type record-rtd record-accessor
-                          get-string-all))
+                          get-string-all bitwise-ior))
   #:export (regexp-split
             hash-keys cat bv-cat get-global-time sanitize-response
             get-local-time string->md5 unsafe-random parse-date write-date
@@ -83,7 +84,7 @@
             ::define did-not-specify-parameter colorize-string-helper
             colorize-string WARN-TEXT ERROR-TEXT REASON-TEXT
             NOTIFY-TEXT STATUS-TEXT get-trigger get-family get-addr request-path
-            response-keep-alive? request-keep-alive?
+            response-keep-alive? request-keep-alive? file->bytevector
             procedure-name->string proper-toplevel gen-content-length
             make-file-sender file-sender? file-sender-size file-sender-thunk
             get-string-all-with-detected-charset make-unstop-exception-handler
@@ -92,7 +93,8 @@
             artanis-sys-response char-predicate handle-upload is-valid-table-name?
             is-guile-compatible-server-core? positive-integer? negative-integer?
             io-exception:peer-is-shutdown? io-exception:out-of-memory?
-            out-of-system-resources? allow-long-live-connection?)
+            out-of-system-resources? allow-long-live-connection?
+            define-c-function ffi-binding free-JS-announcement)
   #:re-export (the-environment
                utf8->string
                bytevector?
@@ -285,10 +287,9 @@
       (or (assoc-ref (request-headers req) 'x-real-ip) "Unknown IP")
       (or (request-ip req) "Unknown IP")))
 
-(define *methods-list* '(HEAD GET POST PUT PATCH DELETE))
+(define *methods-list* '(HEAD GET POST PUT PATCH OPTIONS DELETE))
 (define (allowed-method? method)
-  ;; TODO: check allowed method from config
-  #t)
+  (memq method (get-conf '(server allowedmethods))))
 (define (valid-method? method)
   (if (and (member method *methods-list*) (allowed-method? method))
       method
@@ -318,26 +319,54 @@
 (define-public MAP_FIXED        #x10)
 (define-public MAP_ANONYMOUS    #x20)
 (define-public MAP_UNINITIALIZED 0) ;; don't support map uninitialized
+(define-public MAP_FAILED #xffffffffffffffff)
 
-(define *libc-ffi* (dynamic-link))
+;;void *mmap(void *addr, size_t length, int prot, int flags, int fd, off_t offset)
 (define %mmap
   (pointer->procedure '*
-                      (dynamic-func "mmap" *libc-ffi*)
-                      (list '* size_t int int int size_t) #:return-errno? #t))
+                      (dynamic-func "mmap" (dynamic-link))
+                      (list '* size_t int int int long)
+                      #:return-errno? #t))
+
+;;redefine mmap as
+;;(mmap fd len addr prot flags offset)
+(define* (mmap fd len #:key (addr %null-pointer) (prot (list PROT_READ PROT_WRITE)) (flags (list MAP_SHARED)) (offset 0))
+  (let ((pv (fold bitwise-ior 0 prot))
+        (fv (fold bitwise-ior 0 flags)))
+    (call-with-values
+        (lambda()
+          (%mmap addr len pv fv fd offset))
+      (lambda(ret errno)
+        (cond
+         ((not (= (pointer-address ret) MAP_FAILED)) ret)
+         (else (throw 'artanis-error mmap
+                      "Error: ~a" (strerror errno))))))))
+
 (define %munmap
   (pointer->procedure int
-                      (dynamic-func "munmap" *libc-ffi*)
-                      (list '* size_t) #:return-errno? #t))
-(define* (mmap size #:key (addr %null-pointer) (fd -1) (prot MAP_SHARED)
-               (flags PROT_READ) (offset 0) (bv? #f))
-  (let ((ret (if bv? (lambda (p) (pointer->bytevector p size)) identity)))
-    (ret (%mmap addr size prot flags fd offset))))
-(define (munmap bv/pointer size)
-  (cond
-   ((bytevector? bv/pointer)
-    (%munmap (bytevector->pointer bv/pointer size) size))
-   ((pointer? bv/pointer)
-    (%munmap bv/pointer size))))
+                      (dynamic-func "munmap" (dynamic-link))
+                      (list '* size_t)
+                      #:return-errno? #t))
+
+(define (munmap addr len)
+  (call-with-values
+      (lambda ()
+        (%munmap addr len))
+    (lambda (ret errno)
+      (cond
+       ((>= ret 0) ret)
+       (else (throw 'artanis-error 500 munmap
+                    "Error: ~a" (strerror errno)))))))
+
+(define (file->bytevector filename)
+  (let ((size (stat:size (stat filename)))
+        (port (open-input-file filename)))
+    (let ((bv (pointer->bytevector
+               (mmap (port->fdes port) size
+                     #:prot (list PROT_READ) #:flags (list MAP_SHARED))
+               size)))
+      (close port)
+      bv)))
 
 ;; FIXME: what if len is not even?
 (define (string->byteslist str step base)
@@ -429,9 +458,10 @@
 ;; 1. (> hi (bytevector-length bv))
 ;; 2. (< lo 0) wrap reference
 (define (%bv-slice bv lo hi)
-  (let* ((len (- hi lo))
+  (let* ((len (- hi lo -1))
          (slice ((@ (rnrs) make-bytevector) len)))
-    ((@ (rnrs) bytevector-copy!) bv lo slice 0 len) slice))
+    ((@ (rnrs) bytevector-copy!) bv lo slice 0 len)
+    slice))
 
 ;; NOT SAFE %bytevector-slice for GC, need
 ;;(define (%bytevector-slice bv lo hi)
@@ -877,7 +907,14 @@
           (write-header port)
           (if (eof-object? rl)
               (write '() port)
-              (write (assoc-set! rl url (drop-right meta 1)) port))
+              (write
+               (assoc-set! rl url
+                           (map (lambda (e)
+                                  (if (procedure? e)
+                                      (object->string e)
+                                      e)) meta))
+               port
+               ))
           (flock port LOCK_UN))))))
 
 (define (dump-route-from-cache)
@@ -906,7 +943,7 @@
           (for-each (lambda (r)
                       (let* ((meta (cdr r))
                              (rule (assq-ref meta 'rule))
-                             (method (assq-ref meta 'method)))
+                             (method (and=> (assq-ref meta 'method) car)))
                         (format port "~2t(~a ~s)~%"
                                 (if method method 'get)
                                 (if rule rule (car r)))))
@@ -949,11 +986,11 @@
             lst)
   lst)
 
-(define (subbv->string bv encoding start end)
+(define (subbv->string bv encoding start len)
   (call-with-output-string
     (lambda (port)
       (set-port-encoding! port encoding)
-      (put-bytevector port bv start (- end start)))))
+      (put-bytevector port bv start len))))
 
 (define* (bv-u8-index bv u8 #:optional (time 1))
   (let ((len (bytevector-length bv)))
@@ -996,7 +1033,7 @@
   (bv-read-delimited bv 10 start end))
 
 (define (put-bv port bv from to)
-  (put-bytevector port bv from (- to from 2)))
+  (put-bytevector port bv from (- to from -1)))
 
 ;; TODO: build a char occurence indexing table
 (define (build-bv-lookup-table bv)
@@ -1319,10 +1356,16 @@
                                (current-toplevel) file)))
     (if (file-exists? local-syspage)
         local-syspage
-        (format #f "~a/~a" (get-conf '(server syspage path)) file))))
+        (let ((sys-syspage (format #f "~a/~a" (get-conf '(server syspage path)) file)))
+          (and (file-exists? sys-syspage)
+               sys-syspage)))))
 
-(define (syspage-show file)
-  (bv-cat (get-syspage file) #f))
+(define (syspage-show status)
+  (let* ((file (format #f "~a.html" status))
+         (syspage (get-syspage file)))
+    (if syspage
+        (bv-cat syspage #f)
+        #vu8())))
 
 ;; ENHANCE: use colored output
 (define* (artanis-log blame-who? status mime #:key (port (current-error-port))
@@ -1351,8 +1394,7 @@
 
 (define (render-sys-page blame-who? status request)
   (artanis-log blame-who? status 'text/html #:request request)
-  (let* ((filename (format #f "~a.html" status))
-         (charset (get-conf '(server charset)))
+  (let* ((charset (get-conf '(server charset)))
          (mtime (generate-modify-time (current-time)))
          (guile-compt-serv? (is-guile-compatible-server-core? (get-conf '(server engine))))
          (response
@@ -1360,9 +1402,18 @@
                           #:headers `((server . ,(get-conf '(server info)))
                                       (last-modified . ,mtime)
                                       (content-type . (text/html (charset . ,charset))))))
-         (body (if (resources-collecting?)
-                   #vu8(0) ; Don't open any file since we don't have resources now.
-                   (syspage-show filename))))
+         (body (cond
+                ((resources-collecting?)
+                 #vu8(0)) ; Don't open any file since we don't have resources now
+                ((get-syspage-handler status)
+                 => (lambda (thunk)
+                      (let ((body (thunk)))
+                        (cond
+                         ((string? body)
+                          (string->bytevector body (get-conf '(server charset))))
+                         ((bytevector? body) body)
+                         (else (syspage-show status))))))
+                (else (syspage-show status)))))
     (if guile-compt-serv?
         (values response body)
         (values response body 'exception))))
@@ -1382,9 +1433,8 @@
   (lambda (status)
     (format-status-page/client status request)))
 
-(define (exception-from-server)
-  (lambda (status)
-    (format-status-page/server status)))
+(define (exception-from-server status)
+  (format-status-page/server status))
 
 (define *rf-re* (string->irregex ".*/artanis/artanis/(.*)$"))
 (define (->reasonable-file filename)
@@ -1412,9 +1462,12 @@
          (format port "HTTP ~a~%" (STATUS-TEXT status))
          (format port "Captured in <~a>~%" (WARN-TEXT (->reasonable-file filename)))
          (when subr (format port "Threw in procedure ~a :~%"
-                            (WARN-TEXT (if (procedure? subr)
-                                           (procedure-name->string subr)
-                                           subr))))
+                            (WARN-TEXT (cond
+                                        ((procedure? subr)
+                                         (procedure-name->string subr))
+                                        ((symbol? subr)
+                                         (symbol->string subr))
+                                        (else subr)))))
          (apply format port
                 (REASON-TEXT (string-append "[REASON] " msg))
                 args)
@@ -1525,3 +1578,85 @@
 
 (define (allow-long-live-connection?)
   (> (get-conf '(server timeout)) 0))
+
+(define %%clib (make-parameter (dynamic-link)))
+
+(define-syntax define-c-function
+  (lambda (x)
+    (syntax-case x ()
+      ((_ type name)
+       #`(module-define!
+          (current-module)
+          '#,(datum->syntax #'name (symbol-append '% (syntax->datum #'name)))
+          (pointer->procedure type
+                              (dynamic-func (symbol->string 'name) (%%clib))
+                              '()
+                              #:return-errno? #t)))
+      ((_ type name (para ...))
+       #`(module-define!
+          (current-module)
+          '#,(datum->syntax #'name (symbol-append '% (syntax->datum #'name)))
+          (pointer->procedure type
+                              (dynamic-func (symbol->string 'name) (%%clib))
+                              (list para ...)
+                              #:return-errno? #t))))))
+
+(define-syntax ffi-binding
+  (syntax-rules ()
+    ((_ () body ...)
+     (begin
+       body ...
+       #t))
+    ((_ libname body ...)
+     (parameterize ((%%clib (dynamic-link libname)))
+       body ...
+       #t))))
+
+
+;; Story: When I released GNU Artanis-0.2.1, RMS had asked me if I can
+;;        support LibreJS for freeing Javascript code in the generated code.
+;;        I promised I will. This took almost one year since I was very
+;;        busy on developing a new product (partly use GNU Artanis, of course),
+;;        so it's delayed. Till few months ago, RMS sent mail to ask me
+;;        if I'm ready for it, I realized that the release of GNU Artanis
+;;        is much delayed.
+;;        LibreJS is a way to detect non-free JS code to help you to avoid
+;;        the non-trivial JS code for certain hidden features. Some of them
+;;        are dangerous, some are stolen your privacy, some are just don't
+;;        let you know what's going on.
+;;        GNU Artanis may generate JS code automatically, they're licensed as
+;;        free software definitely. This free-JS-announcement is used to
+;;        license all the generated JS code in the page to GPLv3.
+;;        This announcement is useful when you have LibreJS plugin in your
+;;        browser to detect the JS automatically.
+;;        Although you're free to relicense the code to whatever you prefer,
+;;        I wish you could free the code, no matter what license name it is.
+(define free-JS-announcement
+  "
+    <script>
+       /*
+        @licstart  The following is the entire license notice for the
+        JavaScript code in this page.
+
+        Copyright (C) 2014  Loic J. Duros
+
+        The JavaScript code in this page is free software: you can
+        redistribute it and/or modify it under the terms of the GNU
+        General Public License (GNU GPL) as published by the Free Software
+        Foundation, either version 3 of the License, or (at your option)
+        any later version.  The code is distributed WITHOUT ANY WARRANTY;
+        without even the implied warranty of MERCHANTABILITY or FITNESS
+        FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.
+
+        As additional permission under GNU GPL version 3 section 7, you
+        may distribute non-source (e.g., minimized or compacted) forms of
+        that code without the copy of the GNU GPL normally required by
+        section 4, provided you include this license notice and a URL
+        through which recipients can access the Corresponding Source.
+
+
+        @licend  The above is the entire license notice
+        for the JavaScript code in this page.
+        */
+    </script>
+")
